function [Particles]=BlockParticleFilter(ModelParam,Observation,Algo,State);


Particles.Data=zeros(ModelParam.Dimension,Algo.NoParticles,ModelParam.NoTimeSteps);
Particles.LogWeights=zeros(ModelParam.Dimension,Algo.NoParticles,ModelParam.NoTimeSteps);
Particles.Weights=zeros(ModelParam.Dimension,Algo.NoParticles,ModelParam.NoTimeSteps);
Particles.LogLikelihood=zeros(1,Algo.NoParticles,ModelParam.NoTimeSteps);
Particles.Mean=zeros(ModelParam.Dimension,ModelParam.NoTimeSteps);
Particles.Cov=zeros(ModelParam.Dimension,ModelParam.Dimension,ModelParam.NoTimeSteps);
Particles.NbResampling=0;
Particles.SavedESS=zeros(Algo.NoBlocks,ModelParam.NoTimeSteps);


for t=1:ModelParam.NoTimeSteps
    
    % Pick a Previous Particles - Multinomial Resampling Step 
    if t>1
        for NumBlock=1:Algo.NoBlocks

            CurrentIndex=Algo.IndexForBlocks((1:Algo.BlockSize)+(NumBlock-1)*Algo.BlockSize);
            outIndex = residualR(1:Algo.NoParticles,Particles.Weights(CurrentIndex(1),:,t-1)');

            Particles.Data(CurrentIndex,:,1:t-1)=Particles.Data(CurrentIndex,outIndex,1:t-1);

            Particles.LogWeights(CurrentIndex,:,t-1)=zeros(Algo.BlockSize,Algo.NoParticles);

        end
    end

    
    
    
    % Draw from the Prior Distribution
    
    for j=1:Algo.NoParticles
        
        if t==1
            
            if strcmp(ModelParam.ModelState,'SkewedT')
                W=1./gamrnd(ModelParam.StateDegreeFreedom/2,1/(ModelParam.StateDegreeFreedom/2));
                Particles.Data(:,j,t)=ModelParam.InitialMean+W*ModelParam.StateSkewness+sqrt(W)*ModelParam.StateCovarianceSR*randn(ModelParam.Dimension,1);
            elseif strcmp(ModelParam.ModelState,'Normal')
                Particles.Data(:,j,t)=ModelParam.InitialMean+ModelParam.StateCovarianceSR*randn(ModelParam.Dimension,1);
                
            end            
            
            
            

        else
        
            if strcmp(ModelParam.ModelState,'SkewedT')
                W=1./gamrnd(ModelParam.StateDegreeFreedom/2,1/(ModelParam.StateDegreeFreedom/2));
                Particles.Data(:,j,t)=ModelParam.TransitionMatrix*Particles.Data(:,j,t-1)+W*ModelParam.StateSkewness+sqrt(W)*ModelParam.StateCovarianceSR*randn(ModelParam.Dimension,1);
            elseif  strcmp(ModelParam.ModelState,'Normal')
                Particles.Data(:,j,t)=ModelParam.TransitionMatrix*Particles.Data(:,j,t-1)+ModelParam.StateCovarianceSR*randn(ModelParam.Dimension,1);    
            end 

            
        end
        
        % Computation of the weights
        for NumBlock=1:Algo.NoBlocks
            
            CurrentIndex=Algo.IndexForBlocks((1:Algo.BlockSize)+(NumBlock-1)*Algo.BlockSize);
        

            CurrentIndexObs=Algo.IndexForBlocks((1:Algo.BlockSize)+(NumBlock-1)*Algo.BlockSize);
            
            CurrentBlockLogLikelihood=ComputationLogLikelihoodv1(Observation(CurrentIndexObs,t),ModelParam.ObservationTransition(CurrentIndexObs,CurrentIndex)*Particles.Data(CurrentIndex,j,t),ModelParam);
            if t==1

                Particles.LogWeights(CurrentIndex,j,t)=CurrentBlockLogLikelihood;        

            else

                Particles.LogWeights(CurrentIndex,j,t)=CurrentBlockLogLikelihood+Particles.LogWeights(CurrentIndex,j,t-1);        

            end
        end
        
    end
    
    
    
    for NumBlock=1:Algo.NoBlocks
        CurrentIndex=Algo.IndexForBlocks((1:Algo.BlockSize)+(NumBlock-1)*Algo.BlockSize);
        
        Particles.Weights(CurrentIndex,:,t)=exp(Particles.LogWeights(CurrentIndex,:,t)-max(Particles.LogWeights(CurrentIndex(1),:,t)));
        Particles.Weights(CurrentIndex,:,t)=Particles.Weights(CurrentIndex,:,t)./sum(Particles.Weights(CurrentIndex(1),:,t));
    
        CurrentESS=1/sum((Particles.Weights(CurrentIndex(1),:,t)).^2);
        Particles.SavedESS(NumBlock,t)=CurrentESS;
        
        % Estimation of the Posterior Mean
        Particles.Mean(CurrentIndex,t)=sum(repmat(Particles.Weights(CurrentIndex(1),:,t),Algo.BlockSize,1).*Particles.Data(CurrentIndex,:,t),2);
        
        % Estimation of the Posterior Covariance
        for iCov=1:Algo.NoParticles
            Particles.Cov(CurrentIndex,CurrentIndex,t)=Particles.Cov(CurrentIndex,CurrentIndex,t)+Particles.Weights(CurrentIndex(1),iCov,t)*(Particles.Data(CurrentIndex,iCov,t)-Particles.Mean(CurrentIndex,t))*(Particles.Data(CurrentIndex,iCov,t)-Particles.Mean(CurrentIndex,t))';
        end     
        
    end
    
    
    
    
    
    
    
    
end