%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Compute the drift of the Position Dependent MALA
%   It is dependent 
%   - inverse of the NEgative Hessian
%   - Derivative of the negative Hessian
%   - Possibly other parameters if a transformation has been applied to the
%   Negative Hessian
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function Drift=ComputationDrifPosDepMALA(InvNegHessian,DerivativeNegHessian,Veig,Deig,DeigTrans,AlphaPosDefMatrix,BlockSize,ModelObs)

%%%%%%%%%
% Note to accelerate code owing to the specific properties of the
% independence of the observation (i.e. diagonal matrix)

if isempty(Veig)==1 % 


    % Computation of the drift Term
    Drift=zeros(BlockSize,1);
    
    
    
    for num=1:BlockSize
        
       
        if 1%strcmp(ModelObs,'Poisson')
            TempMatrix=InvNegHessian(:,num)*DerivativeNegHessian(num,1)*InvNegHessian(num,:);%InvNegHessian*DerivativeNegHessian(:,:,num)*InvNegHessian;
        else
            TempMatrix=InvNegHessian*DerivativeNegHessian(:,:,num)*InvNegHessian;
        end

        
        
        Drift=Drift+TempMatrix(:,num);
    end

    Drift=-0.5*Drift;
    
    
else


    % Compute the modified Drift by taking into
    % account the modified NegHessian

    EigenValueVector=diag(Deig);
    MatrixJ=zeros(BlockSize,BlockSize);
    for k=1:BlockSize
        for kk=1:BlockSize
            if k==kk || EigenValueVector(k)==EigenValueVector(kk)
                MatrixJ(k,kk)=coth(AlphaPosDefMatrix*EigenValueVector(k))-AlphaPosDefMatrix*EigenValueVector(k)*csch(AlphaPosDefMatrix*EigenValueVector(k))^2;
            else %Derivative

                MatrixJ(k,kk)=(EigenValueVector(k)*coth(AlphaPosDefMatrix*EigenValueVector(k))-EigenValueVector(kk)*coth(AlphaPosDefMatrix*EigenValueVector(kk)))...
                    /(EigenValueVector(k)-EigenValueVector(kk));
            end
        end
    end 
    MatrixM=inv(DeigTrans)*Veig';

    Drift=zeros(BlockSize,1);
    for num=1:BlockSize
        TempMatrix=MatrixM'*((MatrixJ.*Veig')*(DerivativeNegHessian(:,:,num))*Veig)*MatrixM;
        Drift=Drift+TempMatrix(:,num);
    end

    Drift=-0.5*Drift;




end