%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%   COmpute Neg Hessian Complete Likelihood Current Time
%       p(x_{t,k},y_t|x_{t,\k},x_{1:t-1},y_{1:t-1}
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [NegHessian,Veig,Deig,DeigTrans]=ComputationNegHessian(Obs,MatrixTransformation,BlockSize,CondMean,InvCondCov,RearrangedCurrentState,ModelParam)


CurrentState=RearrangedCurrentState(1:BlockSize,1);
RearrangedObs=MatrixTransformation*Obs;
RearrangedObs=RearrangedObs(1:BlockSize,1);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Computation Gradient Likelihood

if strcmp(ModelParam.ModelObservation,'Poisson')
    


    if 0
    DiagonalTerm=ModelParam.MappingIntensityFunction(CurrentState,ModelParam.MappingIntensityCoeff,ModelParam.MappingIntensityScale)...
        .*ModelParam.MappingIntensityFunctionSecDerivLog(CurrentState,ModelParam.MappingIntensityCoeff,ModelParam.MappingIntensityScale)...
        -ModelParam.MappingIntensityFunctionSecDeriv(CurrentState,ModelParam.MappingIntensityCoeff,ModelParam.MappingIntensityScale);
    end
    
    % If Exponential is used as link function in the Poisson
    DiagonalTerm=-ModelParam.MappingIntensityFunctionSecDeriv(CurrentState,ModelParam.MappingIntensityCoeff,ModelParam.MappingIntensityScale);

    
    
    
    
    
    % In Girolami Double expectation if the process is Gaussian
    % Expectation of the exponential of normal rand variables
    
    if (ModelParam.PoissonNormalPositionDependent==0 && strcmp(ModelParam.ModelState,'Normal')) 


        DiagonalTerm=-ModelParam.MappingIntensityCoeff*exp(CondMean/ModelParam.MappingIntensityScale+diag(inv(InvCondCov))/(ModelParam.MappingIntensityScale^2));
        
       DiagonalTerm(abs(DiagonalTerm)>1e4)=-1e4;
       
       
    end

    DiagonalTerm(isnan(DiagonalTerm))=-1;
    
    HessianLikelihood=diag(DiagonalTerm);
    
    
    
    
elseif strcmp(ModelParam.ModelObservation,'Normal')


    % In case of Normal Likelihood

    
    HessianLikelihood=-(1/ModelParam.VarNoise)*eye(size(CurrentState,1));
    

    
end


% computation of the Hessian of the prior
if strcmp(ModelParam.ModelState,'Normal')
    
    HessianPrior=-InvCondCov;
    
elseif strcmp(ModelParam.ModelState,'SkewedT')
    
    HessianPrior=-ModelParam.InvStateCovSkewedT;
    
end

NegHessian=real(-HessianPrior-HessianLikelihood);



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% If Student T -> Perform a mapping in order to ensure Positive definite
%%%% MAtrix -SoftABS [Betancourt, 2014]
%figure(100);imagesc(NegHessian);title('Before Transformation')

if 0 % Not used in my examples
    
    [NegHessian,Veig,Deig,DeigTrans]=TransformationMatrixPositiveDefinite(real(NegHessian),ModelParam.AlphaPosDefMatrix);

else
    Veig=[]; % Eigenvectors
    Deig=[]; % Matrix with eigenvalues on diagonal
    DeigTrans=[]; % Matrix with eigenvalues on diagonal
end



