function [Kalman]=KalmanFilter(ModelParam,Observation)

 
ObservationCovariance=ModelParam.VarNoise*eye(size(Observation,1));

Kalman.PredictMean=zeros(ModelParam.Dimension,ModelParam.NoTimeSteps);
Kalman.PredictCov=zeros(ModelParam.Dimension,ModelParam.Dimension,ModelParam.NoTimeSteps);
Kalman.UpdateMean=zeros(ModelParam.Dimension,ModelParam.NoTimeSteps);
Kalman.UpdateCov=zeros(ModelParam.Dimension,ModelParam.Dimension,ModelParam.NoTimeSteps);

for t=1:ModelParam.NoTimeSteps
    
    % --- PREDICTION ----
    if t==1
        Kalman.PredictMean(:,t)=ModelParam.InitialMean;
        Kalman.PredictCov(:,:,t)=ModelParam.StateCovariance;
    else
        Kalman.PredictMean(:,t)=ModelParam.TransitionMatrix*Kalman.UpdateMean(:,t-1);
        Kalman.PredictCov(:,:,t)=ModelParam.TransitionMatrix*Kalman.UpdateCov(:,:,t-1)*ModelParam.TransitionMatrix'+ModelParam.StateCovariance;    
    end
    
    % --- UPDATE ----
    Innov=Observation(:,t)-ModelParam.ObservationTransition*Kalman.PredictMean(:,t);
    InnovCov=ModelParam.ObservationTransition*Kalman.PredictCov(:,:,t)*ModelParam.ObservationTransition'+ObservationCovariance;
    Gain=Kalman.PredictCov(:,:,t)*ModelParam.ObservationTransition'*inv(InnovCov);
    
    Kalman.UpdateMean(:,t)=Kalman.PredictMean(:,t)+Gain*Innov;
    Kalman.UpdateCov(:,:,t)=Kalman.PredictCov(:,:,t)-Gain*ModelParam.ObservationTransition*Kalman.PredictCov(:,:,t);
    
end
    