function [Particles]=ParticleFilter(ModelParam,Observation,Algo,State)


Particles.Data=zeros(ModelParam.Dimension,Algo.NoParticles,ModelParam.NoTimeSteps);
Particles.LogWeights=zeros(1,Algo.NoParticles,ModelParam.NoTimeSteps);
Particles.Weights=zeros(1,Algo.NoParticles,ModelParam.NoTimeSteps);
Particles.LogLikelihood=zeros(1,Algo.NoParticles,ModelParam.NoTimeSteps);
Particles.Mean=zeros(ModelParam.Dimension,ModelParam.NoTimeSteps);
Particles.SavedESS=zeros(1,ModelParam.NoTimeSteps);

Particles.NbResampling=0;


for t=1:ModelParam.NoTimeSteps % Iteration over time samples
    
       
    
    
    
    for j=1:Algo.NoParticles % MCMC iteraion for one time sample
        
        if t==1
            
            if strcmp(ModelParam.ModelState,'SkewedT')
                W=1./gamrnd(ModelParam.StateDegreeFreedom/2,1/(ModelParam.StateDegreeFreedom/2));
                Particles.Data(:,j,t)=ModelParam.InitialMean+W*ModelParam.StateSkewness+sqrt(W)*ModelParam.StateCovarianceSR*randn(ModelParam.Dimension,1);
            elseif strcmp(ModelParam.ModelState,'Normal')
                Particles.Data(:,j,t)=ModelParam.InitialMean+ModelParam.StateCovarianceSR*randn(ModelParam.Dimension,1);
                
            end
            
        else
            
            if strcmp(ModelParam.ModelState,'SkewedT')
                W=1./gamrnd(ModelParam.StateDegreeFreedom/2,1/(ModelParam.StateDegreeFreedom/2));
                Particles.Data(:,j,t)=ModelParam.TransitionMatrix*Particles.Data(:,j,t-1)+W*ModelParam.StateSkewness+sqrt(W)*ModelParam.StateCovarianceSR*randn(ModelParam.Dimension,1);
            elseif  strcmp(ModelParam.ModelState,'Normal')
                Particles.Data(:,j,t)=ModelParam.TransitionMatrix*Particles.Data(:,j,t-1)+ModelParam.StateCovarianceSR*randn(ModelParam.Dimension,1);    
            end            
            
        end
        
        
        if t==1
            Particles.LogWeights(1,j,t)=ComputationLogLikelihoodv1(Observation(:,t),ModelParam.ObservationTransition*Particles.Data(:,j,t),ModelParam);
        else
            Particles.LogWeights(1,j,t)=ComputationLogLikelihoodv1(Observation(:,t),ModelParam.ObservationTransition*Particles.Data(:,j,t),ModelParam)...
                +Particles.LogWeights(1,j,t-1);
        end
        
        
    end
    
    Particles.LogWeights(1,:,t)=Particles.LogWeights(1,:,t)-max(Particles.LogWeights(1,:,t));
    
    Particles.Weights(1,:,t)=exp(Particles.LogWeights(1,:,t))./sum(exp(Particles.LogWeights(1,:,t)));
    
    
    % Computation of the ESS
    Neff=1/sum((Particles.Weights(1,:,t)).^2);
    Particles.SavedESS(1,t)=Neff;
    
    
    
    Particles.Mean(:,t)=(Particles.Data(:,:,t)*Particles.Weights(1,:,t)');
    
                
    % Computation of the Covariance    
    Particles.Cov(:,:,t)=zeros(ModelParam.Dimension);
    for iCov=1:Algo.NoParticles
        Particles.Cov(:,:,t)=Particles.Cov(:,:,t)+(Particles.Weights(1,iCov,t))*(Particles.Data(:,iCov,t)-Particles.Mean(:,t))*(Particles.Data(:,iCov,t)-Particles.Mean(:,t))';
    end    
    
     if Algo.ResamplingEachTimeStep || Neff<Algo.ResamplingThreshold%1 % 1 :Resampling each time SIR
    
        
        Particles.NbResampling=Particles.NbResampling+1;
        outIndex = residualR(1:Algo.NoParticles,Particles.Weights(1,:,t)');
    
        Particles.Data(:,:,1:t)=Particles.Data(:,outIndex,1:t);
        
        
        Particles.LogWeights(1,:,t)=zeros(1,Algo.NoParticles);
        
        Particles.Weights(1,:,t)=1/Algo.NoParticles;

        
    end
        
     

           
                
                
end   
